@@ -1,55 +1,72 @@
 "Scripts for generating, solving and sweeping programs"
 from time import time
+import warnings as pywarnings
 import numpy as np
 from ad import adnumber
 from ..nomials import parse_subs
 from ..solution_array import SolutionArray
 from ..keydict import KeyDict
 from ..small_scripts import maybe_flatten
+from ..small_classes import FixedScalar
 from ..exceptions import Infeasible
+from ..globals import SignomialsEnabled
 
 
 def evaluate_linked(constants, linked):
     "Evaluates the values and gradients of linked variables."
-    kdc = KeyDict({k: adnumber(maybe_flatten(v))
+    kdc = KeyDict({k: adnumber(maybe_flatten(v), k)
                    for k, v in constants.items()})
-    kdc.log_gets = True
     kdc_plain = None
-    array_calulated, logged_array_gets = {}, {}
+    array_calulated = {}
+    for key in constants:  # remove gradients from constants
+        key.descr.pop("gradients", None)
     for v, f in linked.items():
         try:
-            if v.veckey and v.veckey.original_fn:
+            if v.veckey and v.veckey.vecfn:
                 if v.veckey not in array_calulated:
-                    ofn = v.veckey.original_fn
-                    array_calulated[v.veckey] = np.array(ofn(kdc))
-                    logged_array_gets[v.veckey] = kdc.logged_gets
-                logged_gets = logged_array_gets[v.veckey]
+                    with SignomialsEnabled():  # to allow use of gpkit.units
+                        vecout = v.veckey.vecfn(kdc)
+                    if not hasattr(vecout, "shape"):
+                        vecout = np.array(vecout)
+                    array_calulated[v.veckey] = vecout
                 out = array_calulated[v.veckey][v.idx]
             else:
-                logged_gets = kdc.logged_gets
-                out = f(kdc)
+                with SignomialsEnabled():  # to allow use of gpkit.units
+                    out = f(kdc)
+            if isinstance(out, FixedScalar):  # to allow use of gpkit.units
+                out = out.value
+            if hasattr(out, "units"):
+                out = out.to(v.units or "dimensionless").magnitude
+            elif out != 0 and v.units:
+                pywarnings.warn(
+                    "Linked function for %s did not return a united value."
+                    " Modifying it to do so (e.g. by using `()` instead of `[]`"
+                    " to access variables) will reduce errors." % v)
+            out = maybe_flatten(out)
+            if not hasattr(out, "x"):
+                constants[v] = out
+                continue  # a new fixed variable, not a calculated one
             constants[v] = out.x
-            v.descr["gradients"] = {}
-            for key in logged_gets:
-                if key.shape:
-                    grad = out.gradient(kdc[key])
-                    v.gradients[key] = np.array(grad)
-                else:
-                    v.gradients[key] = out.d(kdc[key])
+            v.descr["gradients"] = {adn.tag: grad
+                                    for adn, grad in out.d().items()
+                                    if adn.tag}
         except Exception as exception:  # pylint: disable=broad-except
             from .. import settings
             if settings.get("ad_errors_raise", None):
                 raise
-            print("Warning: skipped auto-differentiation of linked variable"
-                  " %s because %s was raised. Set `gpkit.settings"
-                  "[\"ad_errors_raise\"] = True` to raise such Exceptions"
-                  " directly.\n" % (v, repr(exception)))
             if kdc_plain is None:
                 kdc_plain = KeyDict(constants)
             constants[v] = f(kdc_plain)
             v.descr.pop("gradients", None)
-        finally:
-            kdc.logged_gets = set()
+            print("Warning: skipped auto-differentiation of linked variable"
+                  " %s because %s was raised. Set `gpkit.settings"
+                  "[\"ad_errors_raise\"] = True` to raise such Exceptions"
+                  " directly.\n" % (v, repr(exception)))
+            if ("Automatic differentiation not yet supported for <class "
+                    "'gpkit.nomials.math.Monomial'> objects") in str(exception):
+                print("This particular warning may have come from using"
+                      " gpkit.units.* in the function for %s; try using"
+                      " gpkit.ureg.* or gpkit.units.*.units instead." % v)
 
 
 def progify(program, return_attr=None):
@@ -69,7 +86,7 @@ def progify(program, return_attr=None):
             if linked:
                 evaluate_linked(constants, linked)
         prog = program(self.cost, self, constants, **initargs)
-        prog.model = self
+        prog.model = self  # NOTE SIDE EFFECTS
         if return_attr:
             return prog, getattr(prog, return_attr)
         return prog
@@ -79,7 +96,7 @@ def progify(program, return_attr=None):
 def solvify(genfunction):
     "Returns function for making/solving/sweeping a program."
     def solvefn(self, solver=None, *, verbosity=1, skipsweepfailures=False,
-                **solveargs):
+                **kwargs):
         """Forms a mathematical program and attempts to solve it.
 
          Arguments
@@ -91,7 +108,7 @@ def solvify(genfunction):
              Is decremented by one and then passed to programs.
          skipsweepfailures : bool (default False)
              If True, when a solve errors during a sweep, skip it.
-         **solveargs : Passed to solve() call
+         **kwargs : Passed to solve and program init calls
 
          Returns
          -------
@@ -110,11 +127,11 @@ def solvify(genfunction):
         # NOTE SIDE EFFECTS: self.program and self.solution set below
         if sweep:
             run_sweep(genfunction, self, solution, skipsweepfailures,
-                      constants, sweep, linked, solver, verbosity, **solveargs)
+                      constants, sweep, linked, solver, verbosity, **kwargs)
         else:
-            self.program, progsolve = genfunction(self)
-            result = progsolve(solver, verbosity=verbosity, **solveargs)
-            if solveargs.get("process_result", True):
+            self.program, progsolve = genfunction(self, **kwargs)
+            result = progsolve(solver, verbosity=verbosity, **kwargs)
+            if kwargs.get("process_result", True):
                 self.process_result(result)
             solution.append(result)
         solution.to_arrays()
@@ -123,9 +140,9 @@ def solvify(genfunction):
     return solvefn
 
 
-# pylint: disable=too-many-locals,too-many-arguments
+# pylint: disable=too-many-locals,too-many-arguments,too-many-branches,too-many-statements
 def run_sweep(genfunction, self, solution, skipsweepfailures,
-              constants, sweep, linked, solver, verbosity, **solveargs):
+              constants, sweep, linked, solver, verbosity, **kwargs):
     "Runs through a sweep."
     # sort sweeps by the eqstr of their varkey
     sweepvars, sweepvals = zip(*sorted(list(sweep.items()),
@@ -140,7 +157,7 @@ def run_sweep(genfunction, self, solution, skipsweepfailures,
                    for (var, grid) in zip(sweepvars, sweep_grids)}
 
     if verbosity > 0:
-        print("Sweeping over %i solves." % N_passes)
+        print("Sweeping with %i solves:" % N_passes)
         tic = time()
 
     self.program = []
@@ -149,23 +166,28 @@ def run_sweep(genfunction, self, solution, skipsweepfailures,
         constants.update({var: sweep_vect[i]
                           for (var, sweep_vect) in sweep_vects.items()})
         if linked:
-            kdc = KeyDict(constants)
-            constants.update({v: f(kdc) for v, f in linked.items()})
-        program, solvefn = genfunction(self, constants)
+            evaluate_linked(constants, linked)
+        program, solvefn = genfunction(self, constants, **kwargs)
+        program.model = None  # so it doesn't try to debug
         self.program.append(program)  # NOTE: SIDE EFFECTS
         try:
-            result = solvefn(solver, verbosity=verbosity-1, **solveargs)
-            if solveargs.get("process_result", True):
+            if verbosity > 1:
+                print("\nSolve %i:" % i)
+            result = solvefn(solver, verbosity=verbosity-1, **kwargs)
+            if kwargs.get("process_result", True):
                 self.process_result(result)
             solution.append(result)
         except Infeasible as e:
             last_error = e
             if not skipsweepfailures:
                 raise RuntimeWarning(
-                    "Sweep halted! Progress saved to m.program. To skip over"
-                    " such failures, solve with skipsweepfailures=True.") from e
+                    "Solve %i was infeasible; progress saved to m.program."
+                    " To continue sweeping after failures, solve with"
+                    " skipsweepfailures=True." % i) from e
+            if verbosity > 0:
+                print("Solve %i was %s." % (i, e.__class__.__name__))
     if not solution:
-        raise RuntimeWarning("No sweeps solved successfully.") from last_error
+        raise RuntimeWarning("All solves were infeasible.") from last_error
 
     solution["sweepvariables"] = KeyDict()
     ksweep = KeyDict(sweep)
@@ -173,7 +195,14 @@ def run_sweep(genfunction, self, solution, skipsweepfailures,
         if var in ksweep:
             solution["sweepvariables"][var] = val
             del solution["constants"][var]
-        elif var not in linked:
+        elif linked:  # if any variables are linked, we check all of them
+            if hasattr(val[0], "shape"):
+                differences = ((l != val[0]).any() for l in val[1:])
+            else:
+                differences = (l != val[0] for l in val[1:])
+            if not any(differences):
+                solution["constants"][var] = [val[0]]
+        else:
             solution["constants"][var] = [val[0]]
 
     if verbosity > 0:
