@@ -76,15 +76,11 @@ class Signomial(Nomial):
         -------
         Signomial (or Posynomial or Monomial)
         """
-        varset = self.varkeys[var]
-        if len(varset) > 1:
-            raise ValueError("multiple variables %s found for key %s"
-                             % (list(varset), var))
-        if not varset:
+        var = var.key
+        if var not in self.vks:
             diff = NomialMap({EMPTY_HV: 0.0})
             diff.units = None
         else:
-            var, = varset
             diff = self.hmap.diff(var)
         return Signomial(diff, require_positive=False)
 
@@ -117,8 +113,8 @@ class Signomial(Nomial):
         -------
         Monomial (unless self(x0) < 0, in which case a Signomial is returned)
         """
-        x0, _, _ = parse_subs(self.varkeys, x0)  # use only varkey keys
-        psub = self.hmap.sub(x0, self.varkeys, parsedsubs=True)
+        x0, _, _ = parse_subs(self.vks, x0)  # use only varkey keys
+        psub = self.hmap.sub(x0, self.vks, parsedsubs=True)
         if EMPTY_HV not in psub or len(psub) > 1:
             raise ValueError("Variables %s remained after substituting x0=%s"
                              " into %s" % (psub, x0, self))
@@ -126,7 +122,7 @@ class Signomial(Nomial):
         c, exp = c0, HashVector()
         for vk in self.vks:
             val = float(x0[vk])
-            diff, = self.hmap.diff(vk).sub(x0, self.varkeys,
+            diff, = self.hmap.diff(vk).sub(x0, self.vks,
                                            parsedsubs=True).values()
             e = val*diff/c0
             if e:
@@ -167,7 +163,7 @@ class Signomial(Nomial):
         -------
         Returns substituted nomial.
         """
-        return Signomial(self.hmap.sub(substitutions, self.varkeys),
+        return Signomial(self.hmap.sub(substitutions, self.vks),
                          require_positive=require_positive)
 
     def __le__(self, other):
@@ -183,7 +179,7 @@ class Signomial(Nomial):
     def __add__(self, other, rev=False):
         other_hmap = getattr(other, "hmap", None)
         if isinstance(other, Numbers):
-            if not other:  # other is zero
+            if other == 0:
                 return Signomial(self.hmap)
             other_hmap = NomialMap({EMPTY_HV: mag(other)})
             other_hmap.units_of_product(other)
@@ -373,10 +369,11 @@ class ScalarSingleEquationConstraint(SingleEquationConstraint):
 
     def __init__(self, left, oper, right):
         lr = [left, right]
-        self.varkeys = set()
+        self.vks = set()
         for i, sig in enumerate(lr):
             if isinstance(sig, Signomial):
-                self.varkeys.update(sig.vks)
+                for exp in sig.hmap:
+                    self.vks.update(exp)
             else:
                 lr[i] = Signomial(sig)
         from .. import NamedVariables
@@ -470,9 +467,10 @@ class PosynomialInequality(ScalarSingleEquationConstraint):
         "Returns the posys <= 1 representation of this constraint."
         out = []
         for posy in self.unsubbed:
-            fixed, _, _ = parse_subs(posy.varkeys, substitutions, clean=True)
-            hmap = posy.hmap.sub(fixed, posy.varkeys, parsedsubs=True)
-            self.pmap, self.mfm = hmap.mmap(posy.hmap)  # pylint: disable=attribute-defined-outside-init
+            fixed, _, _ = parse_subs(posy.vks, substitutions, clean=True)
+            hmap = posy.hmap.sub(fixed, posy.vks, parsedsubs=True)
+            self.pmap = hmap.mmap(posy.hmap)  # pylint: disable=attribute-defined-outside-init
+            del hmap.expmap, hmap.csmap  # needed only for the mmap call above
             hmap = self._simplify_posy_ineq(hmap, self.pmap, fixed)
             if hmap is not None:
                 if any(c <= 0 for c in hmap.values()):
@@ -490,12 +488,18 @@ class PosynomialInequality(ScalarSingleEquationConstraint):
             for i, mmap in enumerate(self.pmap):
                 for idx, percentage in mmap.items():
                     nu_[idx] += percentage*nu[i]
+            del self.pmap  # not needed after dual has been derived
             if hasattr(self, "const_mmap"):
                 scale = (1-self.const_coeff)/self.const_coeff
                 for idx, percentage in self.const_mmap.items():
                     nu_[idx] += percentage * la*scale
+                del self.const_mmap  # not needed after dual has been derived
             nu = nu_
         self.v_ss = HashVector()
+        if self.parent:
+            self.parent.v_ss = self.v_ss
+        if self.generated_by:
+            self.generated_by.v_ss = self.v_ss
         for nu_i, exp in zip(nu, presub.hmap):
             for vk, x in exp.items():
                 self.v_ss[vk] = nu_i*x + self.v_ss.get(vk, 0)
@@ -513,7 +517,7 @@ class MonomialEquality(PosynomialInequality):
         self.bounded = set()
         self.meq_bounded = {}
         self._las = []
-        if self.unsubbed and len(self.varkeys) > 1:
+        if self.unsubbed and len(self.vks) > 1:
             exp, = self.unsubbed[0].hmap
             for key, e in exp.items():
                 s_e = np.sign(e)
@@ -546,7 +550,7 @@ class MonomialEquality(PosynomialInequality):
     def sens_from_dual(self, la, nu, _):
         "Returns the variable/constraint sensitivities from lambda/nu"
         self._las.append(la)
-        if len(self._las) < 2:
+        if len(self._las) == 1:
             return {}, 0
         la = self._las[0] - self._las[1]
         self._las = []
@@ -595,7 +599,7 @@ class SignomialInequality(ScalarSingleEquationConstraint):
         p_ineq = PosynomialInequality(posy, "<=", negy)
         p_ineq.parent = self
         siglt0_us, = self.unsubbed
-        siglt0_hmap = siglt0_us.hmap.sub(substitutions, siglt0_us.varkeys)
+        siglt0_hmap = siglt0_us.hmap.sub(substitutions, siglt0_us.vks)
         negy_hmap = NomialMap()
         posy_hmaps = defaultdict(NomialMap)
         for o_exp, exp in siglt0_hmap.expmap.items():
@@ -609,8 +613,8 @@ class SignomialInequality(ScalarSingleEquationConstraint):
         self._negysig = Signomial(negy_hmap, require_positive=False)
         self._coeffsigs = {exp: Signomial(hmap, require_positive=False)
                            for exp, hmap in posy_hmaps.items()}
-        self._sigvars = {exp: (list(self._negysig.varkeys)
-                               + list(sig.varkeys))
+        self._sigvars = {exp: (list(self._negysig.vks)
+                               + list(sig.vks))
                          for exp, sig in self._coeffsigs.items()}
         return p_ineq.as_hmapslt1(substitutions)
 
@@ -638,7 +642,7 @@ class SignomialInequality(ScalarSingleEquationConstraint):
             assert not key  # constant
             return value
 
-        var_senss = {}
+        self.v_ss = {}
         invnegy_val = 1/subval(self._negysig)
         for i, nu_i in enumerate(nu):
             mon = self._mons[i]
@@ -651,8 +655,8 @@ class SignomialInequality(ScalarSingleEquationConstraint):
                 var_val = result["variables"][var]
                 sens = (nu_i*inv_mon_val*d_mon_d_var*var_val)
                 assert isinstance(sens, float)
-                var_senss[var] = sens + var_senss.get(var, 0)
-        return var_senss, la
+                self.v_ss[var] = sens + self.v_ss.get(var, 0)
+        return self.v_ss, la
 
     def as_gpconstr(self, x0):
         "Returns GP-compatible approximation at x0"
