@@ -74,9 +74,10 @@ class SolverLog(list):
 
     def write(self, writ):
         "Append and potentially write the new line."
+        if writ[:2] == "b'":
+            writ = writ[2:-1]
         if writ != "\n":
-            writ = writ.rstrip("\n")
-            self.append(str(writ))
+            self.append(writ.rstrip("\n"))
         if self.verbosity > 0:  # pragma: no cover
             self.output.write(writ)
 
@@ -118,7 +119,11 @@ def _append_dict(d_in, d_out):
         if isinstance(v, dict):
             d_out[k] = _append_dict(v, d_out[k])
         else:
-            d_out[k].append(v)
+            try:
+                d_out[k].append(v)
+            except KeyError as e:
+                raise RuntimeWarning("Key `%s` was added after the first sweep."
+                                     % k) from e
     return d_out
 
 
