@@ -1,5 +1,6 @@
 """Defines SolutionArray class"""
 import re
+import json
 import difflib
 from operator import sub
 import warnings as pywarnings
@@ -29,24 +30,70 @@ class SolSavingEnvironment:
     This approximately halves the size of the pickled solution.
     """
 
-    def __init__(self, solarray):
+    def __init__(self, solarray, saveconstraints):
         self.solarray = solarray
         self.attrstore = {}
+        self.saveconstraints = saveconstraints
+        self.constraintstore = None
+
 
     def __enter__(self):
-        for constraint_attr in ["mfm", "pmap", "bounded", "meq_bounded",
-                                "v_ss", "unsubbed", "varkeys"]:
-            store = {}
-            for constraint in self.solarray["sensitivities"]["constraints"]:
-                if getattr(constraint, constraint_attr, None):
-                    store[constraint] = getattr(constraint, constraint_attr)
-                    delattr(constraint, constraint_attr)
-            self.attrstore[constraint_attr] = store
+        if self.saveconstraints:
+            for constraint_attr in ["bounded", "meq_bounded", "vks",
+                                    "v_ss", "unsubbed", "varkeys"]:
+                store = {}
+                for constraint in self.solarray["sensitivities"]["constraints"]:
+                    if getattr(constraint, constraint_attr, None):
+                        store[constraint] = getattr(constraint, constraint_attr)
+                        delattr(constraint, constraint_attr)
+                self.attrstore[constraint_attr] = store
+        else:
+            self.constraintstore = \
+                self.solarray["sensitivities"].pop("constraints")
 
     def __exit__(self, type_, val, traceback):
-        for constraint_attr, store in self.attrstore.items():
-            for constraint, value in store.items():
-                setattr(constraint, constraint_attr, value)
+        if self.saveconstraints:
+            for constraint_attr, store in self.attrstore.items():
+                for constraint, value in store.items():
+                    setattr(constraint, constraint_attr, value)
+        else:
+            self.solarray["sensitivities"]["constraints"] = self.constraintstore
+
+def msenss_table(data, _, **kwargs):
+    "Returns model sensitivity table lines"
+    if "models" not in data.get("sensitivities", {}):
+        return ""
+    data = sorted(data["sensitivities"]["models"].items(),
+                  key=lambda i: ((i[1] < 0.1).all(),
+                                 -np.max(i[1]) if (i[1] < 0.1).all()
+                                 else -round(np.mean(i[1]), 1), i[0]))
+    lines = ["Model Sensitivities", "-------------------"]
+    if kwargs["sortmodelsbysenss"]:
+        lines[0] += " (sorts models in sections below)"
+    previousmsenssstr = ""
+    for model, msenss in data:
+        if not model:  # for now let's only do named models
+            continue
+        if (msenss < 0.1).all():
+            msenss = np.max(msenss)
+            if msenss:
+                msenssstr = "%6s" % ("<1e%i" % np.log10(msenss))
+            else:
+                msenssstr = "  =0  "
+        else:
+            meansenss = round(np.mean(msenss), 1)
+            msenssstr = "%+6.1f" % meansenss
+            deltas = msenss - meansenss
+            if np.max(np.abs(deltas)) > 0.1:
+                deltastrs = ["%+4.1f" % d if abs(d) >= 0.1 else "  - "
+                             for d in deltas]
+                msenssstr += " + [ %s ]" % "  ".join(deltastrs)
+        if msenssstr == previousmsenssstr:
+            msenssstr = " "*len(msenssstr)
+        else:
+            previousmsenssstr = msenssstr
+        lines.append("%s : %s" % (msenssstr, model))
+    return lines + [""] if len(lines) > 3 else []
 
 
 def senss_table(data, showvars=(), title="Variable Sensitivities", **kwargs):
@@ -146,7 +193,7 @@ def constraint_table(data, title, sortbymodel=True, showmodels=True, **_):
             if lines:
                 lines.append(["", ""])
             if model or lines:
-                lines.append([("modelname",), model])
+                lines.append([("newmodelline",), model])
             previous_model = model
         constrstr = constrstr.replace(model, "")
         minlen, maxlen = 25, 80
@@ -173,12 +220,12 @@ def constraint_table(data, title, sortbymodel=True, showmodels=True, **_):
     if not lines:
         lines = [("", "(none)")]
     maxlens = np.max([list(map(len, line)) for line in lines
-                      if line[0] != ("modelname",)], axis=0)
+                      if line[0] != ("newmodelline",)], axis=0)
     dirs = [">", "<"]  # we'll check lengths before using zip
     assert len(list(dirs)) == len(list(maxlens))
     fmts = ["{0:%s%s}" % (direc, L) for direc, L in zip(dirs, maxlens)]
     for i, line in enumerate(lines):
-        if line[0] == ("modelname",):
+        if line[0] == ("newmodelline",):
             linelist = [fmts[0].format(" | "), line[1]]
         else:
             linelist = [fmt.format(s) for fmt, s in zip(fmts, line)]
@@ -188,39 +235,57 @@ def constraint_table(data, title, sortbymodel=True, showmodels=True, **_):
 
 def warnings_table(self, _, **kwargs):
     "Makes a table for all warnings in the solution."
-    title = "Warnings"
-    lines = [title, "="*len(title)]
+    title = "WARNINGS"
+    lines = ["~"*len(title), title, "~"*len(title)]
     if "warnings" not in self or not self["warnings"]:
         return []
-    for wtype in self["warnings"]:
+    for wtype in sorted(self["warnings"]):
         data_vec = self["warnings"][wtype]
+        if len(data_vec) == 0:
+            continue
         if not hasattr(data_vec, "shape"):
-            data_vec = [data_vec]
+            data_vec = [data_vec]  # not a sweep
+        else:
+            all_equal = True
+            for data in data_vec[1:]:
+                eq_i = (data == data_vec[0])
+                if hasattr(eq_i, "all"):
+                    eq_i = eq_i.all()
+                if not eq_i:
+                    all_equal = False
+                    break
+            if all_equal:
+                data_vec = [data_vec[0]]  # warnings identical across sweeps
         for i, data in enumerate(data_vec):
+            if len(data) == 0:
+                continue
+            data = sorted(data, key=lambda l: l[0])  # sort by msg
             title = wtype
             if len(data_vec) > 1:
                 title += " in sweep %i" % i
             if wtype == "Unexpectedly Tight Constraints" and data[0][1]:
-                data = [(-int(1e5*c.relax_sensitivity),
-                         "%+6.2g" % c.relax_sensitivity, id(c), c)
-                        for _, c in data]
+                data = [(-int(1e5*relax_sensitivity),
+                         "%+6.2g" % relax_sensitivity, id(c), c)
+                        for _, (relax_sensitivity, c) in data]
                 lines += constraint_table(data, title, **kwargs)
             elif wtype == "Unexpectedly Loose Constraints" and data[0][1]:
-                data = [(-int(1e5*c.rel_diff),
-                         "%.4g %s %.4g" % c.tightvalues, id(c), c)
-                        for _, c in data]
+                data = [(-int(1e5*rel_diff),
+                         "%.4g %s %.4g" % tightvalues, id(c), c)
+                        for _, (rel_diff, tightvalues, c) in data]
                 lines += constraint_table(data, title, **kwargs)
             else:
                 lines += [title] + ["-"*len(wtype)]
-                for msg, _ in data:
-                    lines += [msg, ""]
-                lines += [""]
-    return lines
+                lines += [msg for msg, _ in data] + [""]
+    if len(lines) == 3:  # just the header
+        return []
+    lines[-1] = "~~~~~~~~"
+    return lines + [""]
 
 
 TABLEFNS = {"sensitivities": senss_table,
             "top sensitivities": topsenss_table,
             "insensitivities": insenss_table,
+            "model sensitivities": msenss_table,
             "tightest constraints": tight_table,
             "loose constraints": loose_table,
             "warnings": warnings_table,
@@ -289,7 +354,8 @@ class SolutionArray(DictOfLists):
     """
     modelstr = ""
     _name_collision_varkeys = None
-    table_titles = {"sweepvariables": "Swept Variables",
+    table_titles = {"choicevariables": "Choice Variables",
+                    "sweepvariables": "Swept Variables",
                     "freevariables": "Free Variables",
                     "constants": "Fixed Variables",  # TODO: change everywhere
                     "variables": "Variables"}
@@ -334,8 +400,9 @@ class SolutionArray(DictOfLists):
 
     # pylint: disable=too-many-locals, too-many-branches, too-many-statements
     def diff(self, other, showvars=None, *,
-             constraintsdiff=True, senssdiff=True, sensstol=0.1,
-             absdiff=False, abstol=0, reldiff=True, reltol=1.0, **tableargs):
+             constraintsdiff=True, senssdiff=False, sensstol=0.1,
+             absdiff=False, abstol=0.1, reldiff=True, reltol=1.0,
+             sortmodelsbysenss=True, **tableargs):
         """Outputs differences between this solution and another
 
         Arguments
@@ -346,9 +413,9 @@ class SolutionArray(DictOfLists):
             if True, show sensitivity differences
         sensstol : float
             the smallest sensitivity difference worth showing
-        abssdiff : boolean
+        absdiff : boolean
             if True, show absolute differences
-        absstol : float
+        abstol : float
             the smallest absolute difference worth showing
         reldiff : boolean
             if True, show relative differences
@@ -359,6 +426,10 @@ class SolutionArray(DictOfLists):
         -------
         str
         """
+        if sortmodelsbysenss:
+            tableargs["sortmodelsbysenss"] = self["sensitivities"]["models"]
+        else:
+            tableargs["sortmodelsbysenss"] = False
         tableargs.update({"hidebelowminval": True, "sortbyvals": True,
                           "skipifempty": False})
         if isinstance(other, Strings):
@@ -369,7 +440,7 @@ class SolutionArray(DictOfLists):
         svars, ovars = self["variables"], other["variables"]
         lines = ["Solution Diff",
                  "=============",
-                 "(positive means the argument is smaller)", ""]
+                 "(argument is the baseline solution)", ""]
         svks, ovks = set(svars), set(ovars)
         if showvars:
             lines[0] += " (for selected variables)"
@@ -383,11 +454,9 @@ class SolutionArray(DictOfLists):
             else:
                 cdiff = ["Constraint Differences",
                          "**********************"]
-                cdiff.extend(difflib.unified_diff(
-                    self.modelstr.split("\n"), other.modelstr.split("\n"),
-                    fromfile="removed in argument", tofile="added in argument",
-                    lineterm="", n=3))
-                cdiff.insert(4, "")
+                cdiff.extend(list(difflib.unified_diff(
+                    other.modelstr.split("\n"), self.modelstr.split("\n"),
+                    lineterm="", n=3))[2:])
                 cdiff += ["", "**********************", ""]
                 lines += cdiff
         if svks - ovks:
@@ -408,7 +477,7 @@ class SolutionArray(DictOfLists):
                                "Relative Differences |above %g%%|" % reltol,
                                valfmt="%+.1f%%  ", vecfmt="%+6.1f%% ",
                                minval=reltol, printunits=False, **tableargs)
-            if lines[-2][:10] == "-"*10:  # nothing larger than sensstol
+            if lines[-2][:10] == "-"*10:  # nothing larger than reltol
                 lines.insert(-1, ("The largest is %+g%%."
                                   % unrolled_absmax(rel_diff.values())))
         if absdiff:
@@ -417,7 +486,7 @@ class SolutionArray(DictOfLists):
                                "Absolute Differences |above %g|" % abstol,
                                valfmt="%+.2g", vecfmt="%+8.2g",
                                minval=abstol, **tableargs)
-            if lines[-2][:10] == "-"*10:  # nothing larger than sensstol
+            if lines[-2][:10] == "-"*10:  # nothing larger than abstol
                 lines.insert(-1, ("The largest is %+g."
                                   % unrolled_absmax(abs_diff.values())))
         if senssdiff:
@@ -434,20 +503,22 @@ class SolutionArray(DictOfLists):
                                   % unrolled_absmax(senss_delta.values())))
         return "\n".join(lines)
 
-    def save(self, filename="solution.pkl", **pickleargs):
+    def save(self, filename="solution.pkl",
+             *, saveconstraints=True, **pickleargs):
         """Pickles the solution and saves it to a file.
 
         Solution can then be loaded with e.g.:
         >>> import pickle
         >>> pickle.load(open("solution.pkl"))
         """
-        with SolSavingEnvironment(self):
+        with SolSavingEnvironment(self, saveconstraints):
             pickle.dump(self, open(filename, "wb"), **pickleargs)
 
-    def save_compressed(self, filename="solution.pgz", **cpickleargs):
+    def save_compressed(self, filename="solution.pgz",
+                        *, saveconstraints=True, **cpickleargs):
         "Pickle a file and then compress it into a file with extension."
         with gzip.open(filename, "wb") as f:
-            with SolSavingEnvironment(self):
+            with SolSavingEnvironment(self, saveconstraints):
                 pickled = pickle.dumps(self, **cpickleargs)
             f.write(pickletools.optimize(pickled))
 
@@ -471,7 +542,7 @@ class SolutionArray(DictOfLists):
             del key.descr["necessarylineage"]
         return names
 
-    def savemat(self, filename="solution.mat", showvars=None,
+    def savemat(self, filename="solution.mat", *, showvars=None,
                 excluded=("unnecessary lineage", "vec")):
         "Saves primal solution as matlab file"
         from scipy.io import savemat
@@ -511,18 +582,40 @@ class SolutionArray(DictOfLists):
                     ", ".join("%s=%s" % (k, v) for (k, v) in key.descr.items()
                               if k not in ["name", "units", "unitrepr",
                                            "idx", "shape", "veckey",
-                                           "value", "original_fn",
+                                           "value", "vecfn",
                                            "lineage", "label"])])
         return pd.DataFrame(rows, columns=cols)
 
-    def savetxt(self, filename="solution.txt", printmodel=True, **kwargs):
+    def savetxt(self, filename="solution.txt", *, printmodel=True, **kwargs):
         "Saves solution table as a text file"
         with open(filename, "w") as f:
             if printmodel:
-                f.write(self.modelstr)
+                f.write(self.modelstr + "\n")
             f.write(self.table(**kwargs))
 
-    def savecsv(self, showvars=None, filename="solution.csv", valcols=5):
+    def savejson(self, filename="solution.json", showvars=None):
+        "Saves solution table as a json file"
+        sol_dict = {}
+        for key in self.name_collision_varkeys():
+            key.descr["necessarylineage"] = True
+        data = self["variables"]
+        if showvars:
+            showvars = self._parse_showvars(showvars)
+            data = {k: data[k] for k in showvars if k in data}
+        # add appropriate data for each variable to the dictionary
+        for k, v in data.items():
+            key = str(k)
+            if isinstance(v, np.ndarray):
+                val = {"v": v.tolist(), "u": k.unitstr()}
+            else:
+                val = {"v": v, "u": k.unitstr()}
+            sol_dict[key] = val
+        for key in self.name_collision_varkeys():
+            del key.descr["necessarylineage"]
+        with open(filename, "w") as f:
+            json.dump(sol_dict, f)
+
+    def savecsv(self, filename="solution.csv", *, valcols=5, showvars=None):
         "Saves primal solution as a CSV sorted by modelname, like the tables."
         data = self["variables"]
         if showvars:
@@ -542,12 +635,14 @@ class SolutionArray(DictOfLists):
             valcols = 1
         if maxspan < valcols:
             valcols = maxspan
-        lines = var_table(data, "", rawlines=True, maxcolumns=valcols)
+        lines = var_table(data, "", rawlines=True, maxcolumns=valcols,
+                          tables=("cost", "sweepvariables", "freevariables",
+                                  "constants", "sensitivities"))
         with open(filename, "w") as f:
             f.write("Model Name,Variable Name,Value(s)" + ","*valcols
                     + "Units,Description\n")
             for line in lines:
-                if line[0] == ("modelname",):
+                if line[0] == ("newmodelline",):
                     f.write(line[1])
                 elif not line[1]:  # spacer line
                     f.write("\n")
@@ -602,9 +697,10 @@ class SolutionArray(DictOfLists):
         return out
 
     def table(self, showvars=(),
-              tables=("cost", "warnings", "sweepvariables", "freevariables",
+              tables=("cost", "warnings", "model sensitivities",
+                      "sweepvariables", "freevariables",
                       "constants", "sensitivities", "tightest constraints"),
-              **kwargs):
+              sortmodelsbysenss=True, **kwargs):
         """A table representation of this SolutionArray
 
         Arguments
@@ -624,6 +720,10 @@ class SolutionArray(DictOfLists):
         -------
         str
         """
+        if sortmodelsbysenss and "sensitivities" in self:
+            kwargs["sortmodelsbysenss"] = self["sensitivities"]["models"]
+        else:
+            kwargs["sortmodelsbysenss"] = False
         varlist = list(self["variables"])
         has_only_one_model = True
         for var in varlist[1:]:
@@ -637,6 +737,9 @@ class SolutionArray(DictOfLists):
         showvars = self._parse_showvars(showvars)
         strs = []
         for table in tables:
+            if "sensitivities" not in self and ("sensitivities" in table or
+                                                "constraints" in table):
+                continue
             if table == "cost":
                 cost = self["cost"]  # pylint: disable=unsubscriptable-object
                 if kwargs.get("latex", None):  # cost is not printed for latex
@@ -693,7 +796,7 @@ def var_table(data, title, *, printunits=True, latex=False, rawlines=False,
               varfmt="%s : ", valfmt="%-.4g ", vecfmt="%-8.3g",
               minval=0, sortbyvals=False, hidebelowminval=False,
               included_models=None, excluded_models=None, sortbymodel=True,
-              maxcolumns=5, skipifempty=True, **_):
+              maxcolumns=5, skipifempty=True, sortmodelsbysenss=None, **_):
     """
     Pretty string representation of a dict of VarKeys
     Iterable values are handled specially (partial printing)
@@ -730,15 +833,18 @@ def var_table(data, title, *, printunits=True, latex=False, rawlines=False,
         if minval and hidebelowminval and getattr(v, "shape", None):
             v[np.abs(v) <= minval] = np.nan
         model = lineagestr(k.lineage) if sortbymodel else ""
+        msenss = -sortmodelsbysenss.get(model, 0) if sortmodelsbysenss else 0
+        if hasattr(msenss, "shape"):
+            msenss = np.mean(msenss)
         models.add(model)
         b = bool(getattr(v, "shape", None))
         s = k.str_without(("lineage", "vec"))
         if not sortbyvals:
-            decorated.append((model, b, (varfmt % s), i, k, v))
+            decorated.append((msenss, model, b, (varfmt % s), i, k, v))
         else:  # for consistent sorting, add small offset to negative vals
             val = np.nanmean(np.abs(v)) - (1e-9 if np.nanmean(v) < 0 else 0)
             sort = (float("%.4g" % -val), k.name)
-            decorated.append((model, sort, b, (varfmt % s), i, k, v))
+            decorated.append((model, sort, msenss, b, (varfmt % s), i, k, v))
     if not decorated and skipifempty:
         return []
     if included_models:
@@ -750,10 +856,10 @@ def var_table(data, title, *, printunits=True, latex=False, rawlines=False,
     decorated.sort()
     previous_model, lines = None, []
     for varlist in decorated:
-        if not sortbyvals:
-            model, isvector, varstr, _, var, val = varlist
+        if sortbyvals:
+            model, _, msenss, isvector, varstr, _, var, val = varlist
         else:
-            model, _, isvector, varstr, _, var, val = varlist
+            msenss, model, isvector, varstr, _, var, val = varlist
         if model not in models:
             continue
         if model != previous_model:
@@ -761,7 +867,7 @@ def var_table(data, title, *, printunits=True, latex=False, rawlines=False,
                 lines.append(["", "", "", ""])
             if model:
                 if not latex:
-                    lines.append([("modelname",), model, "", ""])
+                    lines.append([("newmodelline",), model, "", ""])
                 else:
                     lines.append(
                         [r"\multicolumn{3}{l}{\textbf{" + model + r"}} \\"])
@@ -820,13 +926,13 @@ def var_table(data, title, *, printunits=True, latex=False, rawlines=False,
     if not latex:
         if lines:
             maxlens = np.max([list(map(len, line)) for line in lines
-                              if line[0] != ("modelname",)], axis=0)
+                              if line[0] != ("newmodelline",)], axis=0)
             dirs = [">", "<", "<", "<"]
             # check lengths before using zip
             assert len(list(dirs)) == len(list(maxlens))
             fmts = ["{0:%s%s}" % (direc, L) for direc, L in zip(dirs, maxlens)]
         for i, line in enumerate(lines):
-            if line[0] == ("modelname",):
+            if line[0] == ("newmodelline",):
                 line = [fmts[0].format(" | "), line[1]]
             else:
                 line = [fmt.format(s) for fmt, s in zip(fmts, line)]
