@@ -59,6 +59,38 @@ class TestGP(unittest.TestCase):
                                self.ndig)
         self.assertAlmostEqual(sol["cost"], 2*np.sqrt(2), self.ndig)
 
+    def test_dup_eq_constraint(self):
+        # from https://github.com/convexengineering/gpkit/issues/1551
+        a = Variable("a", 1)
+        b = Variable("b")
+        c = Variable("c", 2)
+        d = Variable("d")
+        z = Variable("z", 0.5)
+
+        # create a simple GP with equality constraints
+        const = [
+            z == b/a,
+            z == d/c,
+        ]
+
+        # simple cost
+        cost = a + b + c + d
+
+        # create a model
+        m = Model(cost, const)
+
+        # solve the first version of the model (solves successfully)
+        m.solve(verbosity=0, solver=self.solver)
+
+        # add a redundant equality constraint
+        m.extend([
+            z == b/a
+        ])
+
+        # solver will fail and attempt to debug
+        m.solve(verbosity=0, solver=self.solver)
+
+
     def test_sigeq(self):
         x = Variable("x")
         y = VectorVariable(1, "y")
@@ -201,11 +233,11 @@ class TestGP(unittest.TestCase):
 
     def test_additive_constants(self):
         x = Variable("x")
-        m = Model(1/x, [1 >= 5*x + 0.5, 1 >= 10*x])
+        m = Model(1/x, [1 >= 5*x + 0.5, 1 >= 5*x])
         m.solve(verbosity=0)
         # pylint: disable=no-member
         gp = m.program  # created by solve()
-        self.assertEqual(gp.cs[1], gp.cs[2])
+        self.assertEqual(gp.cs[1], 2*gp.cs[2])
         self.assertEqual(gp.A.data[1], gp.A.data[2])
 
     def test_zeroing(self):
@@ -288,7 +320,7 @@ class TestSP(unittest.TestCase):
         with SignomialsEnabled():
             m = Model(x, [x+y >= w, x+y <= z/2, y <= x, y >= 1], {z: 3, w: 3})
         r1 = ConstantsRelaxed(m)
-        self.assertEqual(len(r1.varkeys), 8)
+        self.assertEqual(len(r1.vks), 8)
         with self.assertRaises(ValueError):
             _ = Model(x*r1.relaxvars, r1)  # no "prod"
         sp = Model(x*r1.relaxvars.prod()**10, r1).sp(use_pccp=False)
@@ -299,7 +331,7 @@ class TestSP(unittest.TestCase):
             m = Model(x, [x+y >= z, x+y <= z/2, y <= x, y >= 1], {z: 3})
         m.debug(verbosity=0, solver=self.solver)
         r2 = ConstraintsRelaxed(m)
-        self.assertEqual(len(r2.varkeys), 7)
+        self.assertEqual(len(r2.vks), 7)
         sp = Model(x*r2.relaxvars.prod()**10, r2).sp(use_pccp=False)
         cost = sp.localsolve(verbosity=0, solver=self.solver)["cost"]
         self.assertAlmostEqual(cost/1024, 1, self.ndig)
@@ -307,7 +339,7 @@ class TestSP(unittest.TestCase):
             m = Model(x, [x+y >= z, x+y <= z/2, y <= x, y >= 1], {z: 3})
         m.debug(verbosity=0, solver=self.solver)
         r3 = ConstraintsRelaxedEqually(m)
-        self.assertEqual(len(r3.varkeys), 4)
+        self.assertEqual(len(r3.vks), 4)
         sp = Model(x*r3.relaxvar**10, r3).sp(use_pccp=False)
         cost = sp.localsolve(verbosity=0, solver=self.solver)["cost"]
         self.assertAlmostEqual(cost/(32*0.8786796585), 1, self.ndig)
@@ -330,14 +362,14 @@ class TestSP(unittest.TestCase):
                           solver=self.solver, verbosity=0)
 
         with SignomialsEnabled():
-            m = Model(x, Bounded([x + y >= 1], verbosity=0))
+            m = Model(x, Bounded([x + y >= 1]))
         sol = m.localsolve(verbosity=0, solver=self.solver)
         boundedness = sol["boundedness"]
         # depends on solver, platform, whims of the numerical deities
-        if "value near lower bound" in boundedness:  # pragma: no cover
-            self.assertIn(x.key, boundedness["value near lower bound"])
+        if "value near lower bound of 1e-30" in boundedness:  # pragma: no cover
+            self.assertIn(x.key, boundedness["value near lower bound of 1e-30"])
         else:  # pragma: no cover
-            self.assertIn(y.key, boundedness["value near upper bound"])
+            self.assertIn(y.key, boundedness["value near upper bound of 1e+30"])
 
     def test_values_vs_subs(self):
         # Substitutions update method
@@ -386,25 +418,26 @@ class TestSP(unittest.TestCase):
         sys.stdout = stringout = StringIO()
 
         with SignomialsEnabled():
-            m = Model(x, [x + z >= y])
+            m1 = Model(x, [x + z >= y])
         with self.assertRaises(UnnecessarySGP):
-            m.localsolve(verbosity=0, solver=self.solver)
+            m1.localsolve(verbosity=0, solver=self.solver)
         with self.assertRaises(UnboundedGP):
-            m.solve(verbosity=0, solver=self.solver)
+            m1.solve(verbosity=0, solver=self.solver)
 
         with SignomialsEnabled():
-            m = Model(x, [x + y >= z])
-            m.substitutions[y] = 1
-            m.substitutions[z] = 4
-        sol = m.solve(self.solver, verbosity=0)
+            m2 = Model(x, [x + y >= z])
+            m2.substitutions[y] = 1
+            m2.substitutions[z] = 4
+        sol = m2.solve(self.solver, verbosity=0)
         self.assertAlmostEqual(sol["cost"], 3, self.ndig)
 
         sys.stdout = old_stdout
         self.assertEqual(stringout.getvalue(), (
-            "Warning: SignomialConstraint x + z >= y became the tautological"
+            "Warning: SignomialConstraint %s became the tautological"
             " constraint 0 <= 3 + x after substitution.\n"
-            "Warning: SignomialConstraint x + z >= y became the tautological"
-            " constraint 0 <= 3 + x after substitution.\n"))
+            "Warning: SignomialConstraint %s became the tautological"
+            " constraint 0 <= 3 + x after substitution.\n"
+            % (str(m1[0]), str(m1[0]))))
 
     def test_tautological(self):
         x = Variable("x")
@@ -425,10 +458,11 @@ class TestSP(unittest.TestCase):
 
         sys.stdout = old_stdout
         self.assertEqual(stringout.getvalue(), (
-            "Warning: SignomialConstraint x + y >= z became the tautological"
+            "Warning: SignomialConstraint %s became the tautological"
             " constraint 0 <= 1 + x after substitution.\n"
-            "Warning: SignomialConstraint x + 1 >= 0 became the tautological"
-            " constraint 0 <= 1 + x after substitution.\n"))
+            "Warning: SignomialConstraint %s became the tautological"
+            " constraint 0 <= 1 + x after substitution.\n"
+            % (str(m1[0]), str(m2[0]))))
 
     def test_impossible(self):
         x = Variable("x")
@@ -627,19 +661,21 @@ class TestSP(unittest.TestCase):
         with self.assertRaises((DualInfeasible, UnknownInfeasible)):
             m.solve(self.solver, verbosity=0)
         # test one-sided bound
-        m = Model(x*y, Bounded(m, verbosity=0, lower=0.001))
+        m = Model(x*y, Bounded(m, lower=0.001))
         sol = m.solve(self.solver, verbosity=0)
         bounds = sol["boundedness"]
-        self.assertEqual(bounds["sensitive to lower bound"], set([x.key]))
+        self.assertEqual(bounds["sensitive to lower bound of 0.001"],
+                         set([x.key]))
         # end test one-sided bound
-        m = Model(x*y, Bounded(m, verbosity=0))
+        m = Model(x*y, [x*y**1.01 >= 100])
+        m = Model(x*y, Bounded(m))
         sol = m.solve(self.solver, verbosity=0)
         bounds = sol["boundedness"]
         # depends on solver, platform, whims of the numerical deities
-        if "sensitive to upper bound" in bounds:  # pragma: no cover
-            self.assertIn(y.key, bounds["sensitive to upper bound"])
+        if "sensitive to upper bound of 1e+30" in bounds:  # pragma: no cover
+            self.assertIn(y.key, bounds["sensitive to upper bound of 1e+30"])
         else:  # pragma: no cover
-            self.assertIn(x.key, bounds["sensitive to lower bound"])
+            self.assertIn(x.key, bounds["sensitive to lower bound of 1e-30"])
 
 
 class TestModelSolverSpecific(unittest.TestCase):
@@ -729,7 +765,7 @@ class TestModelNoSolve(unittest.TestCase):
     "model tests that don't require a solver"
     def test_modelname_added(self):
         t = Thing(2)
-        for vk in t.varkeys:
+        for vk in t.vks:
             self.assertEqual(vk.lineage, (("Thing", 0),))
 
     def test_modelcontainmentprinting(self):
