@@ -104,7 +104,7 @@ class TestModelSubs(unittest.TestCase):
         x = Variable("x")
         y = Variable("y")
         m = Model(x, [y >= 1], {y: x})
-        with self.assertRaises(ValueError):
+        with self.assertRaises(TypeError):
             m.solve()
 
     def test_quantity_sub(self):
@@ -227,7 +227,8 @@ class TestModelSubs(unittest.TestCase):
     def test_calcconst(self):
         x = Variable("x", "hours")
         t_day = Variable("t_{day}", 12, "hours")
-        t_night = Variable("t_{night}", lambda c: 24 - c[t_day], "hours")
+        t_night = Variable("t_{night}",
+                           lambda c: 1*gpkit.ureg.day - c(t_day), "hours")
         _ = pickle.dumps(t_night)
         m = Model(x, [x >= t_day, x >= t_night])
         sol = m.solve(verbosity=0)
@@ -406,7 +407,7 @@ class TestNomialMapSubs(unittest.TestCase):
         w = Variable("w")
 
         with self.assertRaises(ValueError):
-            z.hmap.sub({z.key: w.key}, varkeys=z.varkeys)
+            z.hmap.sub({z.key: w.key}, varkeys=z.vks)
 
     def test_subinplace_zero(self):
         z = Variable("z")
